@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-shell.c,v 1.29.2.2 2005/07/04 12:47:28 chpe Exp $
+ *  $Id: ephy-embed-shell.c,v 1.32 2005/07/31 18:32:34 chpe Exp $
  */
 
 #include "config.h"
@@ -258,3 +258,16 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 	
 	g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
+
+/**
+ * ephy_embed_shell_get_default:
+ *
+ * Retrieves the default #EphyEmbedShell object
+ *
+ * ReturnValue: the default #EphyEmbedShell
+ **/
+EphyEmbedShell *
+ephy_embed_shell_get_default (void)
+{
+	return embed_shell;
+}
