@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-shell.h,v 1.16 2005/01/24 22:48:04 crispin Exp $
+ *  $Id: ephy-embed-shell.h,v 1.18 2005/07/31 18:32:35 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_SHELL_H
@@ -51,12 +51,16 @@ struct _EphyEmbedShellClass
 {
 	GObjectClass parent_class;
 
+	void	  (* prepare_close)	(EphyEmbedShell *shell);
+
 	/*< private >*/
 	GObject * (* get_embed_single)  (EphyEmbedShell *shell);
 };
 
 GType		   ephy_embed_shell_get_type		(void);
 
+EphyEmbedShell	  *ephy_embed_shell_get_default		(void);
+
 GObject		  *ephy_embed_shell_get_favicon_cache	(EphyEmbedShell *ges);
 
 GObject		  *ephy_embed_shell_get_global_history	(EphyEmbedShell *shell);
@@ -67,6 +71,8 @@ GObject		  *ephy_embed_shell_get_encodings	(EphyEmbedShell *shell);
 
 GObject		  *ephy_embed_shell_get_embed_single	(EphyEmbedShell *shell);
 
+void		   ephy_embed_shell_prepare_close	(EphyEmbedShell *shell);
+
 G_END_DECLS
 
 #endif /* !EPHY_EMBED_SHELL_H */
