@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.c,v 1.64 2005/06/14 19:12:10 crispin Exp $
+ *  $Id: ephy-embed.c,v 1.65 2005/07/29 20:12:55 chpe Exp $
  */
 
 #include "config.h"
@@ -731,19 +731,6 @@ ephy_embed_show_page_certificate (EphyEmbed *embed)
 	iface->show_page_certificate (embed);
 }
 
-/**
- * ephy_embed_activate:
- * @embed: an #EphyEmbed
- *
- * Gives focus to @embed (i.e., Mozilla).
- **/
-void
-ephy_embed_activate (EphyEmbed *embed)
-{
-	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
-	iface->activate (embed);
-}
-
 /**
  * ephy_embed_set_encoding:
  * @embed: an #EphyEmbed
