@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history.c,v 1.59 2005/07/08 21:15:19 chpe Exp $
+ *  $Id: ephy-history.c,v 1.61 2005/08/28 13:54:15 crispin Exp $
  */
 
 #include "config.h"
@@ -34,8 +34,8 @@
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
-#define EPHY_HISTORY_XML_ROOT	 "ephy_history"
-#define EPHY_HISTORY_XML_VERSION "1.0"
+#define EPHY_HISTORY_XML_ROOT	 (const xmlChar *)"ephy_history"
+#define EPHY_HISTORY_XML_VERSION (const xmlChar *)"1.0"
 
 /* how often to save the history, in milliseconds */
 #define HISTORY_SAVE_INTERVAL (5 * 60 * 1000)
@@ -69,6 +69,7 @@ enum
 
 enum
 {
+	ADD_PAGE,
 	VISITED,
 	CLEARED,
 	REDIRECT,
@@ -80,6 +81,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 static void ephy_history_class_init	(EphyHistoryClass *klass);
 static void ephy_history_init		(EphyHistory *history);
 static void ephy_history_finalize	(GObject *object);
+static gboolean impl_add_page           (EphyHistory *histroy, const char *url);
 
 static GObjectClass *parent_class = NULL;
 
@@ -170,6 +172,8 @@ ephy_history_class_init (EphyHistoryClass *klass)
 	object_class->get_property = ephy_history_get_property;
 	object_class->set_property = ephy_history_set_property;
 
+	klass->add_page = impl_add_page;
+
 	g_object_class_install_property (object_class,
 					 PROP_ENABLED,
 					 g_param_spec_boolean ("enabled",
@@ -178,6 +182,17 @@ ephy_history_class_init (EphyHistoryClass *klass)
 							       TRUE,
 							       G_PARAM_READWRITE));
 
+	signals[ADD_PAGE] =
+		g_signal_new ("add_page",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyHistoryClass, add_page),
+			      g_signal_accumulator_true_handled, NULL,
+			      ephy_marshal_BOOLEAN__STRING,
+			      G_TYPE_BOOLEAN,
+			      1,
+			      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
 	signals[VISITED] =
                 g_signal_new ("visited",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -277,7 +292,7 @@ ephy_history_save (EphyHistory *eb)
 	LOG ("Saving history db");
 
 	ret = ephy_node_db_write_to_xml_safe
-		(eb->priv->db, eb->priv->xml_file,
+		(eb->priv->db, (const xmlChar *)eb->priv->xml_file,
 		 EPHY_HISTORY_XML_ROOT,
 		 EPHY_HISTORY_XML_VERSION,
 		 NULL, /* comment */
@@ -657,18 +672,7 @@ ephy_history_host_visited (EphyHistory *eh,
 }
 
 static EphyNode *
-ephy_history_add_host (EphyHistory *eh, EphyNode *page)
-{
-	const char *url;
-
-	url = ephy_node_get_property_string
-		(page, EPHY_NODE_PAGE_PROP_LOCATION);
-
-	return ephy_history_get_host (eh, url);
-}
-
-EphyNode *
-ephy_history_get_host (EphyHistory *eh, const char *url)
+internal_get_host (EphyHistory *eh, const char *url, gboolean create)
 {
 	GnomeVFSURI *vfs_uri = NULL;
 	EphyNode *host = NULL;
@@ -740,7 +744,7 @@ ephy_history_get_host (EphyHistory *eh, const char *url)
 		if (host) break;
 	}
 
-	if (!host)
+	if (!host && create)
 	{
 		host = ephy_node_new (eh->priv->db);
 		ephy_node_signal_connect_object (host,
@@ -773,7 +777,10 @@ ephy_history_get_host (EphyHistory *eh, const char *url)
 		ephy_node_add_child (eh->priv->hosts, host);
 	}
 
-	ephy_history_host_visited (eh, host, now);
+	if (host)
+	{
+		ephy_history_host_visited (eh, host, now);
+	}
 
 	if (vfs_uri)
 	{
@@ -786,6 +793,23 @@ ephy_history_get_host (EphyHistory *eh, const char *url)
 	return host;
 }
 
+EphyNode *
+ephy_history_get_host (EphyHistory *eh, const char *url)
+{
+	return internal_get_host (eh, url, FALSE);
+}
+
+static EphyNode *
+ephy_history_add_host (EphyHistory *eh, EphyNode *page)
+{
+	const char *url;
+
+	url = ephy_node_get_property_string
+		(page, EPHY_NODE_PAGE_PROP_LOCATION);
+
+	return internal_get_host (eh, url, TRUE);
+}
+
 static void
 ephy_history_visited (EphyHistory *eh, EphyNode *node)
 {
@@ -857,22 +881,30 @@ ephy_history_get_page_visits (EphyHistory *gh,
 }
 
 void
-ephy_history_add_page (EphyHistory *eb,
+ephy_history_add_page (EphyHistory *eh,
 		       const char *url)
+{
+	gboolean result = FALSE;
+
+	g_signal_emit (eh, signals[ADD_PAGE], 0, url, &result);
+}
+
+static gboolean
+impl_add_page (EphyHistory *eb, const char *url)
 {
 	EphyNode *bm, *node, *host;
 	GValue value = { 0, };
 
 	if (eb->priv->enabled == FALSE)
 	{
-		return;
+		return FALSE;
 	}
 
 	node = ephy_history_get_page (eb, url);
 	if (node)
 	{
 		ephy_history_visited (eb, node);
-		return;
+		return TRUE;
 	}
 
 	bm = ephy_node_new (eb->priv->db);
@@ -897,6 +929,8 @@ ephy_history_add_page (EphyHistory *eb,
 
 	ephy_node_add_child (host, bm);
 	ephy_node_add_child (eb->priv->pages, bm);
+
+	return TRUE;
 }
 
 EphyNode *
