@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-permission-manager.c,v 1.12 2005/01/06 18:14:41 chpe Exp $
+ *  $Id: ephy-permission-manager.c,v 1.13 2005/04/08 17:02:52 chpe Exp $
  */
 
 #include "config.h"
@@ -147,7 +147,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 		      g_cclosure_marshal_VOID__BOXED,
 		      G_TYPE_NONE,
 		      1,
-		      EPHY_TYPE_PERMISSION_INFO);
+		      EPHY_TYPE_PERMISSION_INFO | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * EphyPermissionManager::permission-changed
@@ -165,7 +165,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 		      g_cclosure_marshal_VOID__BOXED,
 		      G_TYPE_NONE,
 		      1,
-		      EPHY_TYPE_PERMISSION_INFO);
+		      EPHY_TYPE_PERMISSION_INFO | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * EphyPermissionManager::permission-deleted
@@ -183,7 +183,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 		      g_cclosure_marshal_VOID__BOXED,
 		      G_TYPE_NONE,
 		      1,
-		      EPHY_TYPE_PERMISSION_INFO);
+		      EPHY_TYPE_PERMISSION_INFO | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * EphyPermissionManager::permissions-cleared
