@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.94 2005/08/13 14:33:26 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.96 2005/08/27 19:23:42 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -257,7 +257,9 @@ EphyDOMLinkEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
 			NS_ENSURE_TRUE (NS_SUCCEEDED (rv) && docUri, NS_ERROR_FAILURE);
 
 			/* Hide password part */
-			docUri->SetPassword (nsEmbedCString ());
+			nsEmbedCString user;
+			docUri->GetUsername (user);
+			docUri->SetUserPass (user);
 
 			nsEmbedCString resolvedLink;
 			rv = docUri->Resolve (cLink, resolvedLink);
@@ -1327,7 +1329,7 @@ EphyBrowser::GetDocumentType ()
   return type;
 }
 
-#ifdef GTKMOZEMBED_BROKEN_FOCUS
+#ifndef HAVE_GECKO_1_8
 nsresult
 EphyBrowser::FocusActivate ()
 {
@@ -1343,4 +1345,4 @@ EphyBrowser::FocusDeactivate ()
 
 	return mWebBrowserFocus->Deactivate();
 }
-#endif /* GTKMOZEMBED_BROKEN_FOCUS */
+#endif /* !HAVE_GECKO_1_8 */
