@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSDialogs.cpp,v 1.31 2005/07/11 20:41:35 chpe Exp $
+ *  $Id: GtkNSSDialogs.cpp,v 1.32 2005/08/07 21:14:46 crispin Exp $
  */
 
 /*
@@ -358,7 +358,7 @@ GtkNSSDialogs::ConfirmUnknownIssuer (nsIInterfaceRequestor *ctx,
 			       secondary, tertiary);
  
 	res = display_cert_warning_box (ctx, cert, msg, 
- 					_("_Don't show this message again for this site"),
+ 					_("_Trust this security information from now on"),
  					&accept_perm, _("Co_nnect"));
         g_free (tertiary);
         g_free (secondary);
