@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-find.cpp,v 1.1 2005/05/28 21:58:47 chpe Exp $
+ *  $Id: mozilla-embed-find.cpp,v 1.4 2005/08/15 22:22:31 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -28,6 +28,7 @@
 #include "mozilla-embed-find.h"
 #include "ephy-embed-find.h"
 #include "ephy-embed-shell.h"
+#include "ephy-debug.h"
 
 #define MOZILLA_EMBED_FIND_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED_FIND, MozillaEmbedFindPrivate))
 
@@ -82,6 +83,16 @@ impl_find_again (EphyEmbedFind *efind,
 	return priv->find->FindAgain (forward);
 }
 
+static gboolean
+impl_activate_link (EphyEmbedFind *efind,
+		    GdkModifierType mask)
+{
+	MozillaEmbedFind *find = MOZILLA_EMBED_FIND (efind);
+	MozillaEmbedFindPrivate *priv = find->priv;
+
+	return priv->find->ActivateLink (mask);
+}
+
 static void
 ephy_find_iface_init (EphyEmbedFindIface *iface)
 {
@@ -89,6 +100,7 @@ ephy_find_iface_init (EphyEmbedFindIface *iface)
 	iface->set_properties = impl_set_properties;
 	iface->find = impl_find;
 	iface->find_again = impl_find_again;
+	iface->activate_link = impl_activate_link;
 }
 
 static void
@@ -103,6 +115,7 @@ mozilla_embed_find_constructor (GType type, guint n_construct_properties,
 				GObjectConstructParam *construct_params)
 {
 	g_object_ref (embed_shell);
+
 	/* we depend on single because of mozilla initialization */
 	ephy_embed_shell_get_embed_single (embed_shell);
 
