@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-debug.c,v 1.11 2005/07/30 19:41:53 chpe Exp $
+ *  $Id: ephy-debug.c,v 1.11.4.2 2005/08/29 20:36:55 chpe Exp $
  */
 
 #include "config.h"
@@ -23,7 +23,9 @@
 #include "ephy-debug.h"
 
 #include <string.h>
+#ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
+#endif
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -109,11 +111,15 @@ trap_handler (const char *log_domain,
 		}
 		else if (strcmp (ephy_debug_break, "stack") == 0)
 		{
+#ifdef HAVE_EXECINFO_H
 			void *array[MAX_DEPTH];
 			size_t size;
 			
 			size = backtrace (array, MAX_DEPTH);
 			backtrace_symbols_fd (array, size, 2);
+#else
+			g_on_error_stack_trace (g_get_prgname ());
+#endif /* HAVE_EXECINFO_H */
 		}
 		else if (strcmp (ephy_debug_break, "trap") == 0)
 		{
