@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.c,v 1.29 2005/07/03 21:38:54 jframeau Exp $
+ *  $Id: ephy-file-helpers.c,v 1.32 2005/08/20 22:25:48 chpe Exp $
  */
 
 #include "config.h"
@@ -834,13 +834,12 @@ ephy_file_launch_desktop_file (const char *filename,
 
 	for (i = 0; dirs[i] != NULL; i++)
 	{
-		g_print ("Looking in path: %s\n", dirs[i]);
-
 		path = g_build_filename (dirs[i], "applications", filename, NULL);
 
 		if (g_file_test (path, G_FILE_TEST_IS_REGULAR)) break;
 
 		g_free (path);
+		path = NULL;
 	}
 
 	if (path != NULL)
@@ -873,7 +872,7 @@ ephy_file_launch_handler (const char *mime_type,
 	g_return_val_if_fail (address != NULL, FALSE);
 
 	canonical = gnome_vfs_make_uri_canonical (address);
-	g_return_val_if_fail (canonical != NULL, FALSE);
+	if (canonical == NULL) return FALSE;
 
 	if (mime_type != NULL)
 	{
