@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tree-model-sort.c,v 1.14 2005/01/21 19:33:02 jbaayen Exp $
+ *  $Id: ephy-tree-model-sort.c,v 1.15 2005/01/28 22:24:17 chpe Exp $
  */
 
 #include "config.h"
@@ -206,7 +206,7 @@ each_property_get_data_binder (EphyDragEachSelectedItemDataGet iteratee,
 
 		g_return_if_fail (base_data != NULL);
 
-		LOG ("Data get %s (%s)", base_data, extra_data)
+		LOG ("Data get %s (%s)", base_data, extra_data);
 
 		iteratee (base_data, extra_data, data);
 
