@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: ephy-bookmarks-editor.c,v 1.160 2005/01/08 19:41:47 chpe Exp $
+ * $Id: ephy-bookmarks-editor.c,v 1.174 2005/08/08 20:19:56 chpe Exp $
  */
 
 #include "config.h"
@@ -76,9 +76,9 @@ static int n_topic_drag_dest_types = G_N_ELEMENTS (topic_drag_dest_types);
 
 static GtkTargetEntry bmk_drag_types [] =
 {
-        { EPHY_DND_URI_LIST_TYPE,   0, 0 },
-        { EPHY_DND_TEXT_TYPE,       0, 1 },
-        { EPHY_DND_URL_TYPE,        0, 2 }
+        { EPHY_DND_URL_TYPE,        0, 0 },
+        { EPHY_DND_URI_LIST_TYPE,   0, 1 },
+        { EPHY_DND_TEXT_TYPE,       0, 2 }
 };
 static int n_bmk_drag_types = G_N_ELEMENTS (bmk_drag_types);
 
@@ -137,6 +137,8 @@ static void cmd_help_contents		  (GtkAction *action,
 
 #define CONF_BOOKMARKS_VIEW_DETAILS "/apps/epiphany/dialogs/bookmarks_view_details"
 
+#define RESERVED_STRING N_("Remove from this topic")
+
 struct _EphyBookmarksEditorPrivate
 {
 	EphyBookmarks *bookmarks;
@@ -175,7 +177,7 @@ static GtkActionEntry ephy_bookmark_popup_entries [] = {
 	{ "NewTopic", GTK_STOCK_NEW, N_("_New Topic"), "<control>N",
 	  N_("Create a new topic"), 
 	  G_CALLBACK (cmd_add_topic) },
-	{ "OpenInWindow", GTK_STOCK_OPEN, N_("_Open in New Window"), "<control>O",
+	{ "OpenInWindow", GTK_STOCK_OPEN, N_("Open in New _Window"), "<control>O",
 	  N_("Open the selected bookmark in a new window"), 
 	  G_CALLBACK (cmd_open_bookmarks_in_browser) },
 	{ "OpenInTab", STOCK_NEW_TAB, N_("Open in New _Tab"), "<shift><control>O",
@@ -458,7 +460,7 @@ delete_topic_dialog_construct (GtkWindow *parent,
 			       const char *topic)
 {
 	GtkWidget *dialog;
-	
+
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
 					 GTK_DIALOG_MODAL,
 					 GTK_MESSAGE_WARNING,
@@ -540,72 +542,155 @@ show_properties_dialog (EphyBookmarksEditor *editor,
 		(editor->priv->bookmarks, bookmark, GTK_WIDGET (editor));
 }
 
-static void
-add_bookmarks_source (GtkListStore *store,
-		      const char *desc,
-		      const char *dir,
-		      const char *filename,
-		      int max_depth)
+static GSList *
+add_bookmarks_files (const char *dir,
+		     const char *filename,
+		     int max_depth)
 {
-	GtkTreeIter iter;
-	GSList *l;
+	GSList *list;
 	char *path;
 
 	path = g_build_filename (g_get_home_dir (), dir, NULL);
-	l = ephy_file_find  (path, filename, max_depth);
+	list = ephy_file_find  (path, filename, max_depth);
 	g_free (path);
 
-	if (l)
-	{
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 0, desc, 1, l->data, -1);
-	}
-
-	g_slist_foreach (l, (GFunc) g_free, NULL);
-	g_slist_free (l);
+	return list;
 }
 
 static void
-import_from_file_response_cb (GtkDialog *dialog, gint response,
+add_bookmarks_source (const char *file,
+		      GtkListStore *store)
+{
+	GtkTreeIter iter;
+	char **path;
+	char *description = NULL;
+	int len, i;
+
+	path = g_strsplit (file, G_DIR_SEPARATOR_S, -1);
+	g_return_if_fail (path != NULL);
+
+	len = g_strv_length (path);
+
+	for (i = len - 2; i >= 0 && description == NULL; --i)
+	{
+		const char *p = (const char *) path[i];
+
+		g_return_if_fail (p != NULL);
+
+		if (strcmp (p, "firefox") == 0)
+		{
+			description = g_strdup (_("Firefox"));
+		}
+		else if (strcmp (p, ".firefox") == 0)
+		{
+			description = g_strdup (_("Firebird"));
+		}
+		else if (strcmp (p, ".phoenix") == 0)
+		{
+			description = g_strdup (_("Firebird"));
+		}
+		else if (strcmp (p, ".mozilla") == 0)
+		{
+			/* Translators: The %s is the name of a Mozilla profile. */
+			description = g_strdup_printf (_("Mozilla \"%s\" profile"), path[i+1]);
+		}
+		else if (strcmp (p, ".galeon") == 0)
+		{
+			description = g_strdup (_("Galeon"));
+		}
+		else if (strcmp (p, "konqueror") == 0)
+		{
+			description = g_strdup (_("Konqueror"));
+		}
+	}
+
+	if (description != NULL)
+	{
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, description, 1, file, -1);
+
+		g_free (description);
+	}
+
+	g_strfreev (path);
+}
+
+static void
+import_bookmarks (EphyBookmarksEditor *editor,
+		  const char *filename)
+{
+	if (ephy_bookmarks_import (editor->priv->bookmarks, filename) == FALSE)
+	{
+		GtkWidget *dialog;
+		char *basename;
+
+		basename = g_filename_display_basename (filename);
+		dialog = gtk_message_dialog_new (GTK_WINDOW (editor),
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("Import failed"));
+
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Import Failed"));
+		gtk_message_dialog_format_secondary_text
+			(GTK_MESSAGE_DIALOG (dialog),
+			 _("The bookmarks from \"%s\" could not be imported "
+			   "because the file is corrupted or of an "
+			   "unsupported type."),
+			 basename);
+
+		gtk_window_group_add_window (GTK_WINDOW (editor)->group,
+					     GTK_WINDOW (dialog));
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+
+		g_free (basename);
+		gtk_widget_destroy (dialog);
+	}
+}
+
+static void
+import_from_file_response_cb (GtkWidget *dialog,
+			      int response,
 			      EphyBookmarksEditor *editor)
 {
 	char *filename;
 
+	gtk_widget_hide (dialog);
+
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 
 		if (filename != NULL)
 		{
-			ephy_bookmarks_import (editor->priv->bookmarks, filename);			
+			import_bookmarks (editor, filename);			
 
 			g_free (filename);
 		}
 	}
 
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_widget_destroy (dialog);
 }
 
 static void
-import_dialog_response_cb (GtkDialog *dialog, gint response,
+import_dialog_response_cb (GtkDialog *dialog,
+			   gint response,
 			   EphyBookmarksEditor *editor)
 {
 	if (response == GTK_RESPONSE_OK)
 	{
 		GtkTreeIter iter;
-		char *filename;
+		const char *filename;
 		GtkWidget *combo;
 		GtkTreeModel *model;
-		int active;
 		GValue value = { 0, };
 
 		combo = g_object_get_data (G_OBJECT (dialog), "combo_box");
 		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
-		active = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
-		gtk_tree_model_iter_nth_child (model, &iter, NULL, active);
+		gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter);
 		gtk_tree_model_get_value (model, &iter, 1, &value);
-		filename = g_strdup (g_value_get_string (&value));
-		g_value_unset (&value);
+		filename = g_value_get_string (&value);
 
 		if (filename == NULL)
 		{
@@ -619,7 +704,7 @@ import_dialog_response_cb (GtkDialog *dialog, gint response,
 
 			ephy_file_chooser_add_mime_filter
 				(dialog,
-				 _("Firefox/Firebird/Mozilla bookmarks"),
+				 _("Firefox/Mozilla bookmarks"),
 				 "application/x-mozilla-bookmarks", NULL);
 
 			ephy_file_chooser_add_mime_filter
@@ -644,10 +729,10 @@ import_dialog_response_cb (GtkDialog *dialog, gint response,
 		}
 		else
 		{
-			ephy_bookmarks_import (editor->priv->bookmarks, filename);
+			import_bookmarks (editor, filename);
 		}
 
-		g_free (filename);
+		g_value_unset (&value);
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -747,16 +832,17 @@ cmd_bookmarks_import (GtkAction *action,
 	GtkListStore *store;
 	GtkTreeIter iter;
 	GtkTreeModel *sortmodel;
+	GSList *files;
 
 	dialog = gtk_dialog_new_with_buttons (_("Import Bookmarks"),
-					     GTK_WINDOW (editor),
-					     GTK_DIALOG_DESTROY_WITH_PARENT |
-					     GTK_DIALOG_NO_SEPARATOR,
-					     GTK_STOCK_CANCEL,
-					     GTK_RESPONSE_CANCEL,
-					     _("I_mport"),
-					     GTK_RESPONSE_OK,
-					     NULL);
+					      GTK_WINDOW (editor),
+					      GTK_DIALOG_DESTROY_WITH_PARENT |
+					      GTK_DIALOG_NO_SEPARATOR,
+					      GTK_STOCK_CANCEL,
+					      GTK_RESPONSE_CANCEL,
+					      _("I_mport"),
+					      GTK_RESPONSE_OK,
+					      NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
@@ -776,16 +862,16 @@ cmd_bookmarks_import (GtkAction *action,
 
         store = GTK_LIST_STORE (gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING));
 
-	add_bookmarks_source (store, _("Firebird"),
-			      FIREBIRD_BOOKMARKS_DIR, "bookmarks.html", 4);
-	add_bookmarks_source (store, _("Firefox"),
-                              FIREFOX_BOOKMARKS_DIR, "bookmarks.html", 4);
-	add_bookmarks_source (store, _("Galeon"),
-			      GALEON_BOOKMARKS_DIR, "bookmarks.xbel", 0);
-	add_bookmarks_source (store, _("Konqueror"),
-			      KDE_BOOKMARKS_DIR, "bookmarks.xml", 0);
-	add_bookmarks_source (store, _("Mozilla"),
-			      MOZILLA_BOOKMARKS_DIR, "bookmarks.html", 4);
+	files = add_bookmarks_files (FIREFOX_BOOKMARKS_DIR_0, "bookmarks.html", 2);
+	files = g_slist_concat (add_bookmarks_files (FIREFOX_BOOKMARKS_DIR_1, "bookmarks.html", 2), files);
+	/* FIREFOX_BOOKMARKS_DIR_2 is subdir of MOZILLA_BOOKMARKS_DIR, so don't search it twice */
+	files = g_slist_concat (add_bookmarks_files (MOZILLA_BOOKMARKS_DIR, "bookmarks.html", 2), files);
+	files = g_slist_concat (add_bookmarks_files (GALEON_BOOKMARKS_DIR, "bookmarks.xbel", 0), files);
+	files = g_slist_concat (add_bookmarks_files (KDE_BOOKMARKS_DIR, "bookmarks.xml", 0), files);
+	
+	g_slist_foreach (files, (GFunc) add_bookmarks_source, store);
+	g_slist_foreach (files, (GFunc) g_free, NULL);
+	g_slist_free (files);
 
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter, 0, _("File"), 1, NULL, -1);
@@ -1073,13 +1159,14 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	gboolean key_normal = FALSE;
 	gboolean cut, copy, paste, select_all;
 	gboolean can_show_in_bookmarks_bar, show_in_bookmarks_bar = FALSE;
+	gboolean mutable = TRUE;
 	GtkActionGroup *action_group;
 	GtkAction *action;
 	GList *selected;
 	GtkWidget *focus_widget;
 	int num_bmk_selected;
 
-	LOG ("Update menu sensitivity")
+	LOG ("Update menu sensitivity");
 
 	bmk_focus = ephy_node_view_is_target
 		(EPHY_NODE_VIEW (editor->priv->bm_view));
@@ -1145,12 +1232,13 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		id = ephy_node_get_id (node);
 		show_in_bookmarks_bar = ephy_bookmarksbar_model_has_bookmark
 			(editor->priv->tb_model, id);
+		mutable = !ephy_node_get_property_boolean (node, EPHY_NODE_BMK_PROP_IMMUTABLE);
 
 		g_list_free (selected);
 	}
 
-	open_in_window_label = ngettext ("_Open in New Window",
-					 "_Open in New Windows",
+	open_in_window_label = ngettext ("Open in New _Window",
+					 "Open in New _Windows",
 					 num_bmk_selected);
 	open_in_tab_label = ngettext ("Open in New _Tab",
 				      "Open in New _Tabs",
@@ -1167,38 +1255,38 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 
 	open_in_window = (bmk_focus && bmk_selection);
 	open_in_tab = (bmk_focus && bmk_selection);
-	rename = (bmk_focus && single_bmk_selected) ||
+	rename = (bmk_focus && single_bmk_selected && mutable) ||
 		 (key_selection && key_focus && key_normal);
-	delete = (bmk_focus && bmk_selection) ||
+	delete = (bmk_focus && bmk_selection && mutable) ||
 		 (key_selection && key_focus && key_normal);
-	properties = (bmk_focus && single_bmk_selected);
-	can_show_in_bookmarks_bar = (bmk_focus && single_bmk_selected) ||
+	properties = bmk_focus && single_bmk_selected && mutable;
+	can_show_in_bookmarks_bar = (bmk_focus && single_bmk_selected && mutable) ||
 		 (key_selection && key_focus);
 
 	action_group = editor->priv->action_group;
 	action = gtk_action_group_get_action (action_group, "OpenInWindow");
-	g_object_set (action, "sensitive", open_in_window,
-			      "label", open_in_window_label, NULL);
+	gtk_action_set_sensitive (action, open_in_window);
+	g_object_set (action, "label", open_in_window_label, NULL);
 	action = gtk_action_group_get_action (action_group, "OpenInTab");
-	g_object_set (action, "sensitive", open_in_tab,
-			      "label", open_in_tab_label, NULL);
+	gtk_action_set_sensitive (action, open_in_tab);
+	g_object_set (action,  "label", open_in_tab_label, NULL);
 	action = gtk_action_group_get_action (action_group, "Rename");
-	g_object_set (action, "sensitive", rename, NULL);
+	gtk_action_set_sensitive (action, rename);
 	action = gtk_action_group_get_action (action_group, "Delete");
-	g_object_set (action, "sensitive", delete, NULL);
+	gtk_action_set_sensitive (action, delete);
 	action = gtk_action_group_get_action (action_group, "Properties");
-	g_object_set (action, "sensitive", properties, NULL);
+	gtk_action_set_sensitive (action, properties);
 	action = gtk_action_group_get_action (action_group, "Cut");
-	g_object_set (action, "sensitive", cut, NULL);
+	gtk_action_set_sensitive (action, cut);
 	action = gtk_action_group_get_action (action_group, "Copy");
-	g_object_set (action, "sensitive", copy,
-			      "label", copy_label, NULL);
+	gtk_action_set_sensitive (action, copy);
+	g_object_set (action, "label", copy_label, NULL);
 	action = gtk_action_group_get_action (action_group, "Paste");
-	g_object_set (action, "sensitive", paste, NULL);
+	gtk_action_set_sensitive (action, paste);
 	action = gtk_action_group_get_action (action_group, "SelectAll");
-	g_object_set (action, "sensitive", select_all, NULL);
+	gtk_action_set_sensitive (action, select_all);
 	action = gtk_action_group_get_action (action_group, "ShowInBookmarksBar");
-	g_object_set (action, "sensitive", can_show_in_bookmarks_bar, NULL);
+	gtk_action_set_sensitive (action, can_show_in_bookmarks_bar);
 
 	g_signal_handlers_block_by_func
 		(G_OBJECT (GTK_TOGGLE_ACTION (action)),
@@ -1305,10 +1393,17 @@ key_pressed_cb (EphyNodeView *view,
 		GdkEventKey *event,
 		EphyBookmarksEditor *editor)
 {
+	EphyBookmarksEditorPrivate *priv = editor->priv;
+	GtkAction *action;
+
 	if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete)
 	{
-		cmd_delete (NULL, editor);
-		return TRUE;
+		action = gtk_action_group_get_action (priv->action_group, "Delete");
+		if (gtk_action_get_sensitive (action))
+		{
+			cmd_delete (NULL, editor);
+			return TRUE;
+		}
 	} 
 
 	return FALSE;
@@ -1463,7 +1558,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	icon_location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_ICON);
 
-	LOG ("Get favicon for %s", icon_location ? icon_location : "None")
+	LOG ("Get favicon for %s", icon_location ? icon_location : "None");
 
 	if (icon_location)
 	{
@@ -1528,7 +1623,7 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	EphyNode *node;
 	GtkUIManager *ui_merge;
 	GtkActionGroup *action_group;
-	int col_id, details_value;
+	int col_id, url_col_id, title_col_id, details_value;
 
 	ephy_gui_ensure_window_group (GTK_WINDOW (editor));
 
@@ -1598,9 +1693,16 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (key_view),
 					         G_TYPE_STRING, -1,
 						 provide_keyword_uri, editor);
+	ephy_node_view_add_column (EPHY_NODE_VIEW (key_view), _("Topics"),
+				   G_TYPE_STRING,
+				   EPHY_NODE_KEYWORD_PROP_NAME,
+				   EPHY_NODE_VIEW_SHOW_PRIORITY |
+				   EPHY_NODE_VIEW_EDITABLE |
+				   EPHY_NODE_VIEW_SEARCHABLE, NULL, NULL);
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (key_view),
 					   topic_drag_types,
-				           n_topic_drag_types, col_id);
+				           n_topic_drag_types,
+					   col_id, -1);
 	ephy_node_view_enable_drag_dest (EPHY_NODE_VIEW (key_view),
 			                 topic_drag_dest_types,
 			                 n_topic_drag_dest_types);
@@ -1610,12 +1712,6 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 			  "changed",
 			  G_CALLBACK (view_selection_changed_cb),
 			  editor);
-	ephy_node_view_add_column (EPHY_NODE_VIEW (key_view), _("Topics"),
-				   G_TYPE_STRING,
-				   EPHY_NODE_KEYWORD_PROP_NAME,
-				   EPHY_NODE_VIEW_SHOW_PRIORITY |
-				   EPHY_NODE_VIEW_EDITABLE |
-				   EPHY_NODE_VIEW_SEARCHABLE, NULL);
 	ephy_node_view_set_priority (EPHY_NODE_VIEW (key_view),
 				     EPHY_NODE_KEYWORD_PROP_PRIORITY);
 	ephy_node_view_set_sort (EPHY_NODE_VIEW (key_view), G_TYPE_STRING,
@@ -1666,24 +1762,20 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	/* Bookmarks View */
 	bm_view = ephy_node_view_new (node, editor->priv->bookmarks_filter);
 	add_focus_monitor (editor, bm_view);
-	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (bm_view),
-					         G_TYPE_STRING,
-					         EPHY_NODE_BMK_PROP_LOCATION,
-						 NULL, NULL);
-	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (bm_view),
-					   bmk_drag_types,
-				           n_bmk_drag_types,
-					   col_id);
-	editor->priv->title_col = ephy_node_view_add_column
+	title_col_id = ephy_node_view_add_column
 				  (EPHY_NODE_VIEW (bm_view), _("Title"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_TITLE,
 				   EPHY_NODE_VIEW_EDITABLE |
 				   EPHY_NODE_VIEW_SEARCHABLE,
-				   provide_favicon);
-	editor->priv->address_col = ephy_node_view_add_column
+				   provide_favicon, &(editor->priv->title_col));
+	url_col_id = ephy_node_view_add_column
 				  (EPHY_NODE_VIEW (bm_view), _("Address"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_LOCATION,
-				   0, NULL);
+				   0, NULL, &(editor->priv->address_col));
+	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (bm_view),
+					   bmk_drag_types,
+				           n_bmk_drag_types,
+					   url_col_id, title_col_id);
 	ephy_node_view_set_sort (EPHY_NODE_VIEW (bm_view), G_TYPE_STRING,
 				 EPHY_NODE_BMK_PROP_TITLE, GTK_SORT_ASCENDING);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), bm_view);
