@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.143 2005/07/10 19:15:42 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.158 2005/08/21 19:04:24 chpe Exp $
  */
 
 #include "config.h"
@@ -72,26 +72,34 @@ struct _EphyTabPrivate
 {
 	char *status_message;
 	char *link_message;
-	char *icon_address;
 	char *address;
+	char *typed_address;
 	char *title;
-	int load_percent;
-	gboolean visibility;
-	gboolean load_status;
-	EphyTabAddressExpire address_expire;
+	char *loading_title;
+	char *icon_address;
+	GdkPixbuf *icon;
 	int cur_requests;
 	int total_requests;
 	int width;
 	int height;
 	float zoom;
-	gboolean setting_zoom;
-	EphyEmbedSecurityLevel security_level;
 	GSList *hidden_popups;
 	GSList *shown_popups;
 	EphyTabNavigationFlags nav_flags;
 	EphyEmbedDocumentType document_type;
 	guint idle_resize_handler;
 
+	gint8 load_percent;
+	/* Flags */
+	guint is_blank : 1;
+	guint visibility : 1;
+	guint is_loading : 1;
+	guint is_setting_zoom : 1;
+	EphyEmbedSecurityLevel security_level;
+	/* guint security_level : 3; ? */
+	EphyTabAddressExpire address_expire;
+	/* guint address_expire : 2; ? */
+
 	/* File watch */
 	GnomeVFSMonitorHandle *monitor;
 	guint reload_scheduled_id;	
@@ -108,6 +116,7 @@ enum
 	PROP_ADDRESS,
 	PROP_DOCUMENT_TYPE,
 	PROP_ICON,
+	PROP_ICON_ADDRESS,
 	PROP_LOAD_PROGRESS,
 	PROP_LOAD_STATUS,
 	PROP_MESSAGE,
@@ -116,6 +125,7 @@ enum
 	PROP_HIDDEN_POPUP_COUNT,
 	PROP_POPUPS_ALLOWED,
 	PROP_TITLE,
+	PROP_TYPED_ADDRESS,
 	PROP_VISIBLE,
 	PROP_ZOOM
 };
@@ -129,8 +139,6 @@ typedef struct
 static GObjectClass *parent_class = NULL;
 
 /* internal functions, accessible only from this file */
-static void	ephy_tab_set_icon_address	(EphyTab *tab,
-						 const char *location);
 static void	ephy_tab_set_load_status	(EphyTab *tab,
 						 gboolean status);
 static void	ephy_tab_set_link_message	(EphyTab *tab,
@@ -204,14 +212,17 @@ ephy_tab_set_property (GObject *object,
 	switch (prop_id)
 
 	{
-		case PROP_ADDRESS:
-			ephy_tab_set_location (tab, g_value_get_string (value),
-					       EPHY_TAB_ADDRESS_EXPIRE_NOW);
+		case PROP_TYPED_ADDRESS:
+			ephy_tab_set_typed_address (tab, g_value_get_string (value),
+						    EPHY_TAB_ADDRESS_EXPIRE_NOW);
 			break;
 		case PROP_POPUPS_ALLOWED:
-			ephy_tab_set_popups_allowed
-				(tab, g_value_get_boolean (value));
+			ephy_tab_set_popups_allowed (tab, g_value_get_boolean (value));
 			break;
+		case PROP_ICON_ADDRESS:
+			ephy_tab_set_icon_address (tab, g_value_get_string (value));
+			break;
+		case PROP_ADDRESS:
 		case PROP_DOCUMENT_TYPE:
 		case PROP_ICON:
 		case PROP_LOAD_PROGRESS:
@@ -235,32 +246,36 @@ ephy_tab_get_property (GObject *object,
 		       GParamSpec *pspec)
 {
 	EphyTab *tab = EPHY_TAB (object);
+	EphyTabPrivate *priv = tab->priv;
 
 	switch (prop_id)
 	{
 		case PROP_ADDRESS:
-			g_value_set_string (value, tab->priv->address);
+			g_value_set_string (value, priv->address);
 			break;
 		case PROP_DOCUMENT_TYPE:
-			g_value_set_enum (value, tab->priv->document_type);
+			g_value_set_enum (value, priv->document_type);
 			break;
 		case PROP_ICON:
-			g_value_set_string (value, tab->priv->icon_address);
+			g_value_set_object (value, priv->icon);
+			break;
+		case PROP_ICON_ADDRESS:
+			g_value_set_string (value, priv->icon_address);
 			break;
 		case PROP_LOAD_PROGRESS:
-			g_value_set_int (value, tab->priv->load_percent);
+			g_value_set_int (value, priv->load_percent);
 			break;
 		case PROP_LOAD_STATUS:
-			g_value_set_boolean (value, tab->priv->load_status);
+			g_value_set_boolean (value, priv->is_loading);
 			break;
 		case PROP_MESSAGE:
 			g_value_set_string (value, ephy_tab_get_status_message (tab));
 			break;
 		case PROP_NAVIGATION:
-			g_value_set_flags (value, tab->priv->nav_flags);
+			g_value_set_flags (value, priv->nav_flags);
 			break;
 		case PROP_SECURITY:
-			g_value_set_enum (value, tab->priv->security_level);
+			g_value_set_enum (value, priv->security_level);
 			break;
 		case PROP_HIDDEN_POPUP_COUNT:
 			g_value_set_int (value, popup_blocker_n_hidden (tab));
@@ -270,13 +285,16 @@ ephy_tab_get_property (GObject *object,
 				(value, ephy_tab_get_popups_allowed (tab));
 			break;
 		case PROP_TITLE:
-			g_value_set_string (value, tab->priv->title);
+			g_value_set_string (value, priv->title);
+			break;
+		case PROP_TYPED_ADDRESS:
+			g_value_set_string (value, ephy_tab_get_typed_address (tab));
 			break;
 		case PROP_VISIBLE:
-			g_value_set_boolean (value, tab->priv->visibility);
+			g_value_set_boolean (value, priv->visibility);
 			break;
 		case PROP_ZOOM:
-			g_value_set_float (value, tab->priv->zoom);
+			g_value_set_float (value, priv->zoom);
 			break;
 	}
 }
@@ -331,7 +349,7 @@ ephy_tab_grab_focus (GtkWidget *widget)
 {
 	EphyTab *tab = EPHY_TAB (widget);
 
-	ephy_embed_activate (ephy_tab_get_embed (tab));
+	gtk_widget_grab_focus (GTK_WIDGET (ephy_tab_get_embed (tab)));
 }
 
 static EphyWindow *
@@ -368,7 +386,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Address",
 							      "The tab's address",
 							      "",
-							      G_PARAM_READWRITE));
+							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
 					 PROP_DOCUMENT_TYPE,
@@ -381,11 +399,19 @@ ephy_tab_class_init (EphyTabClass *class)
 
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
-					 g_param_spec_string ("icon",
+					 g_param_spec_object ("icon",
+							      "Icon",
+							      "The tab icon's",
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_ICON_ADDRESS,
+					 g_param_spec_string ("icon-address",
 							      "Icon address",
 							      "The tab icon's address",
 							      NULL,
-							      G_PARAM_READABLE));
+							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
 	g_object_class_install_property (object_class,
 					 PROP_LOAD_PROGRESS,
@@ -457,6 +483,14 @@ ephy_tab_class_init (EphyTabClass *class)
 							      _("Blank page"),
 							      G_PARAM_READABLE));
 
+	g_object_class_install_property (object_class,
+					 PROP_TYPED_ADDRESS,
+					 g_param_spec_string ("typed-address",
+							      "Typed Address",
+							      "The typed address",
+							      "",
+							      G_PARAM_READABLE | G_PARAM_WRITABLE));
+
 	g_object_class_install_property (object_class,
 					 PROP_VISIBLE,
 					 g_param_spec_boolean ("visibility",
@@ -475,7 +509,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							     1.0,
 							     G_PARAM_READABLE));
 
-	g_type_class_add_private (object_class, sizeof(EphyTabPrivate));
+	g_type_class_add_private (object_class, sizeof (EphyTabPrivate));
 }
 
 static void
@@ -783,8 +817,16 @@ ephy_tab_finalize (GObject *object)
 	EphyTab *tab = EPHY_TAB (object);
 	EphyTabPrivate *priv = tab->priv;
 
+	if (priv->icon != NULL)
+	{
+		g_object_unref (priv->icon);
+		priv->icon = NULL;
+	}
+
 	g_free (priv->title);
+	g_free (priv->loading_title);
 	g_free (priv->address);
+	g_free (priv->typed_address);
 	g_free (priv->icon_address);
 	g_free (priv->link_message);
 	g_free (priv->status_message);
@@ -824,6 +866,85 @@ let_me_resize_hack (EphyTab *tab)
 	return FALSE;
 }
 
+
+static char *
+get_title_from_address (const char *address)
+{
+	GnomeVFSURI *uri;
+	char *title;
+
+	if (address == NULL) return NULL;
+		
+	uri = gnome_vfs_uri_new (address);
+	if (uri == NULL) return g_strdup (address);
+		
+	title = gnome_vfs_uri_to_string (uri,
+			GNOME_VFS_URI_HIDE_USER_NAME |
+			GNOME_VFS_URI_HIDE_PASSWORD |
+			GNOME_VFS_URI_HIDE_HOST_PORT |
+			GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
+			GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
+	gnome_vfs_uri_unref (uri);
+
+	return title;
+}
+
+static void
+ephy_tab_set_loading_title (EphyTab *tab,
+			    const char *title,
+			    gboolean is_address)
+{
+	EphyTabPrivate *priv = tab->priv;
+	char *freeme = NULL;
+
+	g_free (priv->loading_title);
+	priv->loading_title = NULL;
+
+	if (is_address)
+	{
+		title = freeme = get_title_from_address (title);	
+	}
+
+	if (title != NULL && title[0] != '\0')
+	{
+		/* translators: %s here is the address of the web page */
+		priv->loading_title = g_strdup_printf (_("Loading “%s”..."), title);
+	}
+	else
+	{
+		priv->loading_title = g_strdup (_("Loading..."));
+	}
+
+	g_free (freeme);
+}
+
+/* consumes |address| */
+static void
+ephy_tab_set_address (EphyTab *tab,
+		      char *address)
+{
+	EphyTabPrivate *priv = tab->priv;
+	GObject *object = G_OBJECT (tab);
+
+	g_free (priv->address);
+	priv->address = address;
+
+	priv->is_blank = address == NULL ||
+			 strcmp (address, "about:blank") == 0;
+
+	if (priv->is_loading &&
+	    priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW &&
+	    priv->typed_address != NULL)
+	{
+		g_free (priv->typed_address);
+		priv->typed_address = NULL;
+
+		g_object_notify (object, "typed-address");
+	}
+
+	g_object_notify (object, "address");
+}
+
 /* Public functions */
 
 /**
@@ -845,7 +966,9 @@ ephy_tab_set_load_status (EphyTab *tab, gboolean status)
 {
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
-	tab->priv->load_status = status;
+	status = status != FALSE;
+
+	tab->priv->is_loading = status;
 
 	g_object_notify (G_OBJECT (tab), "load-status");
 }
@@ -881,7 +1004,7 @@ ephy_tab_get_load_status (EphyTab *tab)
 {
 	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
 
-	return tab->priv->load_status;
+	return tab->priv->is_loading;
 }
 
 static void
@@ -1043,46 +1166,75 @@ ephy_tab_get_visibility (EphyTab *tab)
 	return tab->priv->visibility;
 }
 
+static void
+ephy_tab_load_icon (EphyTab *tab)
+{
+	EphyTabPrivate *priv = tab->priv;
+	EphyEmbedShell *shell;
+	EphyFaviconCache *cache;
+
+	if (priv->icon_address == NULL || priv->icon != NULL) return;
+
+	shell = ephy_embed_shell_get_default ();
+	cache = EPHY_FAVICON_CACHE (ephy_embed_shell_get_favicon_cache (shell));
+
+	/* ephy_favicon_cache_get returns a reference already */
+	priv->icon = ephy_favicon_cache_get (cache, priv->icon_address);
+
+	g_object_notify (G_OBJECT (tab), "icon");
+}
+
 static void
 ephy_tab_icon_cache_changed_cb (EphyFaviconCache *cache,
 				const char *address,
 				EphyTab *tab)
 {
+	EphyTabPrivate *priv = tab->priv;
+
 	g_return_if_fail (address != NULL);
 
 	/* is this for us? */
-	if (tab->priv->icon_address != NULL &&
-	    strcmp (tab->priv->icon_address, address) == 0)
+	if (priv->icon_address != NULL &&
+	    strcmp (priv->icon_address, address) == 0)
 	{
-		/* notify */
-		g_object_notify (G_OBJECT (tab), "icon");
+		ephy_tab_load_icon (tab);
 	}
 }
 
-static void
-ephy_tab_set_icon_address (EphyTab *tab, const char *address)
+void
+ephy_tab_set_icon_address (EphyTab *tab,
+			   const char *address)
 {
+	GObject *object = G_OBJECT (tab);
+	EphyTabPrivate *priv = tab->priv;
 	EphyBookmarks *eb;
 	EphyHistory *history;
 
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	g_free (priv->icon_address);
+	priv->icon_address = g_strdup (address);
 
-	g_free (tab->priv->icon_address);
+	if (priv->icon != NULL)
+	{
+		g_object_unref (priv->icon);
+		priv->icon = NULL;
 
-	tab->priv->icon_address = g_strdup (address);
+		g_object_notify (object, "icon");
+	}
 
-	if (tab->priv->icon_address)
+	if (priv->icon_address)
 	{
 		eb = ephy_shell_get_bookmarks (ephy_shell);
 		history = EPHY_HISTORY
 			(ephy_embed_shell_get_global_history (embed_shell));
-		ephy_bookmarks_set_icon (eb, tab->priv->address,
-				         tab->priv->icon_address);
-		ephy_history_set_icon (history, tab->priv->address,
-				       tab->priv->icon_address);
+		ephy_bookmarks_set_icon (eb, priv->address,
+				         priv->icon_address);
+		ephy_history_set_icon (history, priv->address,
+				       priv->icon_address);
+
+		ephy_tab_load_icon (tab);
 	}
 
-	g_object_notify (G_OBJECT (tab), "icon");
+	g_object_notify (object, "icon-address");
 }
 
 static void
@@ -1172,11 +1324,12 @@ ephy_tab_update_file_monitor (EphyTab *tab,
 {
 	EphyTabPrivate *priv = tab->priv;
 	GnomeVFSMonitorHandle *handle = NULL;
-	GnomeVFSURI *uri;
 	gboolean local;
 
-	if (priv->address != NULL && address != NULL &&
+	if (priv->monitor != NULL &&
+	    priv->address != NULL && address != NULL &&
 	    strcmp (priv->address, address) == 0)
+	
 	{
 		/* same address, no change needed */
 		return;
@@ -1184,12 +1337,7 @@ ephy_tab_update_file_monitor (EphyTab *tab,
 
 	ephy_tab_file_monitor_cancel (tab);
 
-	uri = gnome_vfs_uri_new (address);
-	if (uri == NULL) return;
-
-	local = gnome_vfs_uri_is_local (uri);
-	gnome_vfs_uri_unref (uri);
-
+	local = g_str_has_prefix (address, "file://");
 	if (local == FALSE) return;
 	
 	if (gnome_vfs_monitor_add (&handle, address,
@@ -1203,13 +1351,34 @@ ephy_tab_update_file_monitor (EphyTab *tab,
 	}
 }
 
+/**
+ * ephy_tab_get_icon:
+ * @tab: an #EphyTab
+ *
+ * Returns the tab's site icon as a #GdkPixbuf,
+ * or %NULL if it is not available.
+ *
+ * Return value: a the tab's site icon
+ **/
+GdkPixbuf *
+ephy_tab_get_icon (EphyTab *tab)
+{
+	EphyTabPrivate *priv;
+
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
+
+	priv = tab->priv;
+
+	return priv->icon;
+}
+
 /**
  * ephy_tab_get_icon_address:
  * @tab: an #EphyTab
  *
- * Returns a URL which points to @tab's favicon.
+ * Returns a URL which points to @tab's site icon.
  *
- * Return value: a URL to @tab's favicon
+ * Return value: the URL of @tab's site icon
  **/
 const char *
 ephy_tab_get_icon_address (EphyTab *tab)
@@ -1236,45 +1405,72 @@ ephy_tab_link_message_cb (EphyEmbed *embed,
 	ephy_tab_set_link_message (tab, ephy_embed_get_link_message (embed));
 }
 
+static gboolean
+ephy_tab_open_uri_cb (EphyEmbed *embed,
+		      const char *uri,
+		      EphyTab *tab)
+{
+	EphyTabPrivate *priv = tab->priv;
+
+	/* Set the address here if we have a blank page.
+	 * See bug #147840.
+	 */
+	if (priv->is_blank)
+	{
+		ephy_tab_set_address (tab, g_strdup (uri));
+		ephy_tab_set_loading_title (tab, uri, TRUE);
+	}
+
+	/* allow load to proceed */
+	return FALSE;
+}
+
 static void
 ephy_tab_address_cb (EphyEmbed *embed,
 		     const char *address,
 		     EphyTab *tab)
 {
-	const char *uv_address;
-	char *freeme = NULL;
+	GObject *object = G_OBJECT (tab);
 
 	LOG ("ephy_tab_address_cb tab %p address %s", tab, address);
 
+	g_object_freeze_notify (object);
+
 	/* do this up here so we still have the old address around */
 	ephy_tab_update_file_monitor (tab, address);
 
 	/* Do not expose about:blank to the user, an empty address
 	   bar will do better */
-	if (address && strcmp (address, "about:blank") == 0)
+	if (address == NULL || address[0] == '\0' ||
+	    strcmp (address, "about:blank") == 0)
 	{
-		uv_address = "";
+		ephy_tab_set_address (tab, NULL);
+		ephy_tab_set_title (tab, embed, NULL);
 	}
 	else
 	{
-		uv_address = freeme = ephy_embed_get_location (embed, TRUE);
-	}
+		char *embed_address;
 
-	if (tab->priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW)
-	{
-		ephy_tab_set_location (tab, uv_address, EPHY_TAB_ADDRESS_EXPIRE_NOW);
+		/* we do this to get rid of an eventual password in the URL */
+		embed_address = ephy_embed_get_location (embed, TRUE);
+		ephy_tab_set_address (tab, embed_address);
+		ephy_tab_set_loading_title (tab, embed_address, TRUE);
 	}
 
 	ephy_tab_set_link_message (tab, NULL);
 	ephy_tab_set_icon_address (tab, NULL);
 	ephy_tab_update_navigation_flags (tab, embed);
 
-	g_free (freeme);
+	g_object_notify (object, "title");
+
+	g_object_thaw_notify (object);
 }
 
 static void
 ephy_tab_content_change_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 {
+	EphyTabPrivate *priv = tab->priv;
+
 	/* restore zoom level */
 	if (address_has_web_scheme (address))
 	{
@@ -1297,9 +1493,9 @@ ephy_tab_content_change_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 		current_zoom = ephy_embed_get_zoom (embed);
 		if (zoom != current_zoom)
 		{
-			tab->priv->setting_zoom = TRUE;
+			priv->is_setting_zoom = TRUE;
 			ephy_embed_set_zoom (embed, zoom);
-			tab->priv->setting_zoom = FALSE;
+			priv->is_setting_zoom = FALSE;
 		}
 	}
 
@@ -1329,7 +1525,7 @@ ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 
 	ephy_tab_set_zoom (tab, zoom);
 
-	if (tab->priv->setting_zoom)
+	if (tab->priv->is_setting_zoom)
 	{
 		return;
 	}
@@ -1362,13 +1558,19 @@ ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 }
 
 static void
-ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
+ephy_tab_title_cb (EphyEmbed *embed,
+		   EphyTab *tab)
 {
-	char *title;
+	EphyTabPrivate *priv = tab->priv;
+	GObject *object = G_OBJECT (tab);
 
-	title = ephy_embed_get_title (embed);
+	g_object_freeze_notify (object);
 
-	ephy_tab_set_title (tab, embed, title);
+	ephy_tab_set_title (tab, embed, ephy_embed_get_title (embed));
+
+	ephy_tab_set_loading_title (tab, priv->title, FALSE);
+
+	g_object_thaw_notify (object);
 }
 
 static int
@@ -1480,12 +1682,15 @@ build_progress_from_requests (EphyTab *tab, EphyEmbedNetState state)
 static void
 ensure_page_info (EphyTab *tab, EphyEmbed *embed, const char *address)
 {
-	if ((tab->priv->address == NULL || tab->priv->address[0] == '\0') &&
-	    tab->priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW)
+	EphyTabPrivate *priv = tab->priv;
+
+	if ((priv->address == NULL || priv->address[0] == '\0') &&
+	    priv->address_expire == EPHY_TAB_ADDRESS_EXPIRE_NOW)
         {
-		ephy_tab_set_location (tab, address, EPHY_TAB_ADDRESS_EXPIRE_NOW);
+		ephy_tab_set_address (tab, g_strdup (address));
 	}
 
+	/* FIXME huh?? */
 	if (tab->priv->title == NULL || tab->priv->title[0] == '\0')
 	{
 		ephy_tab_set_title (tab, embed, NULL);
@@ -1498,26 +1703,49 @@ ephy_tab_net_state_cb (EphyEmbed *embed,
 		       EphyEmbedNetState state,
 		       EphyTab *tab)
 {
+	EphyTabPrivate *priv = tab->priv;
+
 	update_net_state_message (tab, uri, state);
 
 	if (state & EPHY_EMBED_STATE_IS_NETWORK)
 	{
 		if (state & EPHY_EMBED_STATE_START)
 		{
-			tab->priv->total_requests = 0;
-			tab->priv->cur_requests = 0;
-			ensure_page_info (tab, embed, uri);
+			GObject *object = G_OBJECT (tab);
+
+			g_object_freeze_notify (object);
+
+			priv->total_requests = 0;
+			priv->cur_requests = 0;
 
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, TRUE);
 			ephy_tab_update_navigation_flags (tab, embed);
+
+			ensure_page_info (tab, embed, uri);
+
+			g_object_notify (object, "title");
+
+			g_object_thaw_notify (object);
 		}
 		else if (state & EPHY_EMBED_STATE_STOP)
 		{
+			GObject *object = G_OBJECT (tab);
+
+			g_object_freeze_notify (object);
+
 			ephy_tab_set_load_percent (tab, 100);
 			ephy_tab_set_load_status (tab, FALSE);
 			ephy_tab_update_navigation_flags (tab, embed);
-			tab->priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+
+			g_free (priv->loading_title);
+			priv->loading_title = NULL;
+
+			priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+
+			g_object_notify (object, "title");
+
+			g_object_thaw_notify (object);
 		}
 	}
 
@@ -1754,24 +1982,32 @@ ephy_tab_security_change_cb (EphyEmbed *embed, EphyEmbedSecurityLevel level,
 static void
 ephy_tab_init (EphyTab *tab)
 {
+	EphyTabPrivate *priv;
 	GObject *embed;
 	EphyFaviconCache *cache;
 
 	LOG ("EphyTab initialising %p", tab);
 
-	tab->priv = EPHY_TAB_GET_PRIVATE (tab);
+	priv = tab->priv = EPHY_TAB_GET_PRIVATE (tab);
 
 	tab->priv->total_requests = 0;
 	tab->priv->cur_requests = 0;
 	tab->priv->width = -1;
 	tab->priv->height = -1;
 	tab->priv->load_percent = 0;
-	tab->priv->load_status = FALSE;
+	tab->priv->is_loading = FALSE;
 	tab->priv->security_level = EPHY_EMBED_STATE_IS_UNKNOWN;
 	tab->priv->document_type = EPHY_EMBED_DOCUMENT_HTML;
 	tab->priv->zoom = 1.0;
-	tab->priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
-	tab->priv->title = g_strdup ("");
+	priv->title = NULL;
+	priv->is_blank = TRUE;
+	priv->icon_address = NULL;
+	priv->icon = NULL;
+	priv->address = NULL;
+	priv->typed_address = NULL;
+	priv->title = NULL;
+	priv->loading_title = NULL;
+	priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
 
 	embed = ephy_embed_factory_new_object (EPHY_TYPE_EMBED);
 	g_assert (embed != NULL);
@@ -1785,6 +2021,9 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect_object (embed, "ge_document_type",
 				 G_CALLBACK (ephy_tab_document_type_cb),
 				 tab, 0);
+	g_signal_connect_object (embed, "open_uri",
+				 G_CALLBACK (ephy_tab_open_uri_cb),
+				 tab, 0);
 	g_signal_connect_object (embed, "ge_location",
 				 G_CALLBACK (ephy_tab_address_cb),
 				 tab, 0);
@@ -1949,52 +2188,25 @@ ephy_tab_set_title (EphyTab *tab,
 		    EphyEmbed *embed,
 		    char *title)
 {
- 	if (title == NULL || title[0] == '\0')
+	EphyTabPrivate *priv = tab->priv;
+
+	if (!priv->is_blank && (title == NULL || title[0] == '\0'))
 	{
-		GnomeVFSURI *uri = NULL;
-		char *address;
 
 		g_free (title);
-		title = NULL;
-
-		address = ephy_embed_get_location (embed, TRUE);
-
-		if (address != NULL)
-		{
-			uri = gnome_vfs_uri_new (address);
-		}
-
-		if (uri != NULL)
-		{
-			title = gnome_vfs_uri_to_string (uri,
-					GNOME_VFS_URI_HIDE_USER_NAME |
-					GNOME_VFS_URI_HIDE_PASSWORD |
-					GNOME_VFS_URI_HIDE_HOST_PORT |
-					GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
-					GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
-			gnome_vfs_uri_unref (uri);
-			g_free (address);
-		}
-		else if (address != NULL &&
-			 strncmp (address, "about:blank", 11) != 0)
-		{
-			title = address;
-		}
-		else
-		{
-			g_free (address);
-		}
+		title = get_title_from_address (priv->address);
 
 		/* Fallback */
 		if (title == NULL || title[0] == '\0')
 		{
 			g_free (title);
-			title = g_strdup (_("Blank page"));
+			title = NULL;
+			priv->is_blank = TRUE;
 		}
 	}
 
-	g_free (tab->priv->title);
-	tab->priv->title = title;
+	g_free (priv->title);
+	priv->title = title;
 
 	g_object_notify (G_OBJECT (tab), "title");
 }
@@ -2005,18 +2217,57 @@ ephy_tab_set_title (EphyTab *tab,
  *
  * Returns the title of the web page loaded in @tab.
  *
- * Return value: @tab's loaded web page's title
+ * Return value: @tab's loaded web page's title. Will never be %NULL.
  **/
 const char *
 ephy_tab_get_title (EphyTab *tab)
 {
+	EphyTabPrivate *priv;
+	const char *title = "";
+
 	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
-	return tab->priv->title;
+	priv = tab->priv;
+
+	if (priv->is_blank)
+	{
+		title = _("Blank page");
+	}
+	else if (priv->is_loading &&
+		 priv->loading_title != NULL)
+	{
+		title = priv->loading_title;
+	}
+	else
+	{
+		title = priv->title;
+	}
+
+	return title != NULL ? title : "";
 }
 
 /**
- * ephy_tab_get_location:
+ * ephy_tab_get_address:
+ * @tab: an #EphyTab
+ *
+ * Returns the address of the currently loaded page.
+ *
+ * Return value: @tab's address. Will never be %NULL.
+ **/
+const char *
+ephy_tab_get_address (EphyTab *tab)
+{
+	EphyTabPrivate *priv;
+
+	g_return_val_if_fail (EPHY_IS_TAB (tab), "");
+
+	priv = tab->priv;
+
+	return priv->address ? priv->address : "about:blank";
+}
+
+/**
+ * ephy_tab_get_typed_address:
  * @tab: an #EphyTab
  *
  * Returns the text that @tab's #EphyWindow will display in its location toolbar
@@ -2030,42 +2281,47 @@ ephy_tab_get_title (EphyTab *tab)
  * Return value: @tab's #EphyWindow's location entry when @tab is selected
  **/
 const char *
-ephy_tab_get_location (EphyTab *tab)
+ephy_tab_get_typed_address (EphyTab *tab)
 {
-	g_return_val_if_fail (EPHY_IS_TAB (tab), "");
+	EphyTabPrivate *priv;
 
-	return tab->priv->address;
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
+
+	priv = tab->priv;
+
+	return priv->typed_address;
 }
 
 /**
- * ephy_tab_set_location:
+ * ephy_tab_set_typed_address:
  * @tab: an #EphyTab
- * @address: a string (ideally a URL)
- * @expire: beats me
- *
- * DO NOT USE
- */
+ * @address: the new typed address, or %NULL to clear it
+ * @expire: when to expire this address_expire
+ * 
+ * Sets the text that @tab's #EphyWindow will display in its location toolbar
+ * entry when @tab is selected.
+ **/
 void
-ephy_tab_set_location (EphyTab *tab,
-		       const char *address,
-		       EphyTabAddressExpire expire)
+ephy_tab_set_typed_address (EphyTab *tab,
+			    const char *address,
+			    EphyTabAddressExpire expire)
 {
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	EphyTabPrivate *priv = tab->priv;
 
-	g_free (tab->priv->address);
-	tab->priv->address = g_strdup (address);
+	g_free (priv->typed_address);
+	priv->typed_address = g_strdup (address);
 
 	if (expire == EPHY_TAB_ADDRESS_EXPIRE_CURRENT &&
-	    !tab->priv->load_status)
+	    !priv->is_loading)
 	{
-		tab->priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
+		priv->address_expire = EPHY_TAB_ADDRESS_EXPIRE_NOW;
 	}
 	else
 	{
-		tab->priv->address_expire = expire;
+		priv->address_expire = expire;
 	}
 
-	g_object_notify (G_OBJECT (tab), "address");
+	g_object_notify (G_OBJECT (tab), "typed-address");
 }
 
 static void
