@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004, 2005 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.h,v 1.23 2005/01/06 18:14:42 chpe Exp $
+ *  $Id: ephy-tab.h,v 1.29 2005/08/10 18:14:09 chpe Exp $
  */
 
 #ifndef EPHY_TAB_H
@@ -24,6 +24,7 @@
 
 #include "ephy-embed.h"
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkbin.h>
 
 G_BEGIN_DECLS
@@ -41,7 +42,7 @@ typedef struct _EphyTabPrivate	EphyTabPrivate;
 
 typedef enum
 {
-	EPHY_TAB_NAV_UP	= 1 << 0,
+	EPHY_TAB_NAV_UP		= 1 << 0,
 	EPHY_TAB_NAV_BACK	= 1 << 1,
 	EPHY_TAB_NAV_FORWARD	= 1 << 2
 } EphyTabNavigationFlags;
@@ -66,23 +67,22 @@ struct _EphyTabClass
 	GtkBinClass parent_class;
 };
 
-GType			ephy_tab_address_expire_get_type	(void);
-
-GType			ephy_tab_navigation_flags_get_type	(void);
-
 GType			ephy_tab_get_type		(void);
 
 EphyTab	               *ephy_tab_new			(void);
 
-GObject                *ephy_tab_get_action		(EphyTab *tab);
-
 EphyEmbed              *ephy_tab_get_embed		(EphyTab *tab);
 
 EphyTab		       *ephy_tab_for_embed		(EphyEmbed *embed);
 
 EphyEmbedDocumentType	ephy_tab_get_document_type	(EphyTab *tab);
 
-const char             *ephy_tab_get_icon_address	(EphyTab *tab);
+GdkPixbuf	       *ephy_tab_get_icon		(EphyTab *tab);
+
+const char	       *ephy_tab_get_icon_address	(EphyTab *tab);
+
+void			ephy_tab_set_icon_address	(EphyTab *tab,
+							 const char *address);
 
 gboolean		ephy_tab_get_load_status	(EphyTab *tab);
 
@@ -91,11 +91,13 @@ const char             *ephy_tab_get_link_message	(EphyTab *tab);
 
 int			ephy_tab_get_load_percent	(EphyTab *tab);
 
-void			ephy_tab_set_location		(EphyTab *tab,
-							 const char *location,
-							 EphyTabAddressExpire expire);
+const char             *ephy_tab_get_address		(EphyTab *tab);
 
-const char             *ephy_tab_get_location		(EphyTab *tab);
+const char	       *ephy_tab_get_typed_address	(EphyTab *tab);
+
+void			ephy_tab_set_typed_address	(EphyTab *tab,
+							 const char *address,
+							 EphyTabAddressExpire expire);
 
 EphyTabNavigationFlags	ephy_tab_get_navigation_flags	(EphyTab *tab);
 
@@ -105,6 +107,10 @@ void			ephy_tab_get_size		(EphyTab *tab,
 							 int *width,
 							 int *height);
 
+void			ephy_tab_set_size		(EphyTab *tab,
+							 int width,
+							 int height);
+
 const char             *ephy_tab_get_status_message	(EphyTab *tab);
 
 const char             *ephy_tab_get_title		(EphyTab *tab);
