@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-toolbar.c,v 1.16 2005/08/08 20:19:56 chpe Exp $
+ *  $Id: ephy-toolbar.c,v 1.17 2005/08/10 18:14:09 chpe Exp $
  */
 
 #include "config.h"
@@ -224,7 +224,8 @@ sync_user_input_cb (EphyLocationAction *action,
 	address = ephy_location_action_get_address (action);
 
 	priv->updating_address = TRUE;
-	ephy_tab_set_typed_address (tab, address);
+	ephy_tab_set_typed_address (tab, address,
+				    EPHY_TAB_ADDRESS_EXPIRE_CURRENT);
 	priv->updating_address = FALSE;
 }
 
