@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.293 2005/07/04 12:22:41 chpe Exp $
+ *  $Id: ephy-window.c,v 1.305 2005/08/26 13:11:40 chpe Exp $
  */
 
 #include "config.h"
@@ -49,7 +49,6 @@
 #include "ephy-tabs-menu.h"
 #include "ephy-stock-icons.h"
 #include "ephy-extension.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-link.h"
 #include "ephy-gui.h"
 #include "ephy-notebook.h"
@@ -200,6 +199,8 @@ static const GtkActionEntry ephy_menu_entries [] = {
 	{ "ViewStop", GTK_STOCK_STOP, N_("_Stop"), "Escape",
 	  N_("Stop current data transfer"),
 	  G_CALLBACK (window_cmd_view_stop) },
+	{ "ViewAlwaysStop", GTK_STOCK_STOP, N_("_Stop"), "Escape",
+	  NULL, G_CALLBACK (window_cmd_view_stop) },
 	{ "ViewReload", GTK_STOCK_REFRESH, N_("_Reload"), "<control>R",
 	  N_("Display the latest content of the current page"),
 	  G_CALLBACK (window_cmd_view_reload) },
@@ -282,7 +283,7 @@ static const GtkToggleActionEntry ephy_menu_toggle_entries [] =
 	/* File Menu */
 
 	{ "FileWorkOffline", NULL, N_("_Work Offline"), NULL,
-	  N_("Toggle network status"),
+	  N_("Switch to offline mode"),
 	  G_CALLBACK (window_cmd_file_work_offline), FALSE },
 
 	/* View Menu */
@@ -422,6 +423,8 @@ struct _EphyWindowPrivate
 	guint help_message_cid;
 	EphyEmbedChrome chrome;
 	guint idle_resize_handler;
+	EphyEmbedEvent *context_event;
+	guint idle_worker;
 
 	guint browse_with_caret_notifier_id;
 	guint allow_popups_notifier_id;
@@ -705,6 +708,35 @@ ephy_window_key_press_event (GtkWidget *widget,
 	guint mask = gtk_accelerator_get_default_mod_mask ();
 	char *accel = NULL;
 
+	/* Handle ESC here instead of an action callback, so we can
+	 * handle it differently in the location entry, and we can
+	 * stop even when the page is not loading (to stop animations).
+	 */
+	if (event->keyval == GDK_Escape && (event->state & mask) == 0)
+	{
+		GtkWidget *widget;
+		EphyEmbed *embed;
+		gboolean handled = FALSE;
+
+		widget = gtk_window_get_focus (GTK_WINDOW (window));
+
+		if (GTK_IS_WIDGET (widget))
+		{
+			handled = gtk_widget_event (widget, (GdkEvent*)event);
+		}
+
+		embed = ephy_window_get_active_embed (window);
+		if (handled == FALSE && embed != NULL)
+		{
+			gtk_widget_grab_focus (GTK_WIDGET (embed));
+			ephy_embed_stop_load (embed);
+
+			handled = TRUE;
+		}
+
+		return handled;
+	}
+		
 	/* Don't activate menubar in ppv mode, or in lockdown mode */
 	if (window->priv->ppv_mode || eel_gconf_get_boolean (CONF_LOCKDOWN_HIDE_MENUBAR))
 	{
@@ -744,8 +776,11 @@ ephy_window_key_press_event (GtkWidget *widget,
 }
 
 static gboolean
-ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *window)
+ephy_window_delete_event (GtkWidget *widget,
+			  GdkEventAny *event)
 {
+	EphyWindow *window = EPHY_WINDOW (widget);
+	EphyWindowPrivate *priv = window->priv;
 	EphyTab *modified_tab = NULL;
 	GList *tabs, *l;
 	gboolean modified = FALSE;
@@ -753,7 +788,7 @@ ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *win
 	/* Workaround a crash when closing a window while in print preview mode. See
 	 * mozilla bug #241809
 	 */
-	if (window->priv->ppv_mode)
+	if (priv->ppv_mode)
 	{
 		EphyEmbed *embed;
 
@@ -799,6 +834,11 @@ ephy_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, EphyWindow *win
 	gtk_widget_hide (widget);
 
 	/* proceed with window close */
+	if (GTK_WIDGET_CLASS (parent_class)->delete_event)
+	{
+		return GTK_WIDGET_CLASS (parent_class)->delete_event (widget, event);
+	}
+
 	return FALSE;
 }
 
@@ -1116,17 +1156,11 @@ setup_ui_manager (EphyWindow *window)
 static void
 sync_tab_address (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 {
-	const char *address;
-
 	if (window->priv->closing) return;
 
-	address = ephy_tab_get_location (tab);
-	if (address == NULL)
-	{
-		address = "";
-	}
-
-	ephy_toolbar_set_location (window->priv->toolbar, address);
+	ephy_toolbar_set_location (window->priv->toolbar,
+				   ephy_tab_get_address (tab),
+				   ephy_tab_get_typed_address (tab));
 }
 
 static void
@@ -1168,33 +1202,19 @@ sync_tab_document_type (EphyTab *tab,
 }
 
 static void
-sync_tab_icon (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
+sync_tab_icon (EphyTab *tab,
+	       GParamSpec *pspec,
+	       EphyWindow *window)
 {
-	const char *address;
-	EphyFaviconCache *cache;
-	GdkPixbuf *pixbuf = NULL;
+	EphyWindowPrivate *priv = window->priv;
+	GdkPixbuf *icon;
 
-	if (window->priv->closing) return;
+	if (priv->closing) return;
 
-	cache = EPHY_FAVICON_CACHE
-		(ephy_embed_shell_get_favicon_cache
-			(EPHY_EMBED_SHELL (ephy_shell)));
+	icon = ephy_tab_get_icon (tab);
 
-	address = ephy_tab_get_icon_address (tab);
-
-	if (address)
-	{
-		pixbuf = ephy_favicon_cache_get (cache, address);
-	}
-
-	gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
-
-	ephy_toolbar_set_favicon (window->priv->toolbar, address);
-
-	if (pixbuf)
-	{
-		g_object_unref (pixbuf);
-	}
+	gtk_window_set_icon (GTK_WINDOW (window), icon);
+	ephy_toolbar_set_favicon (priv->toolbar, icon);
 }
 
 static void
@@ -1433,19 +1453,15 @@ sync_tab_load_status (EphyTab *tab,
 }
 
 static void
-sync_tab_title (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
+sync_tab_title (EphyTab *tab,
+		GParamSpec *pspec,
+		EphyWindow *window)
 {
-	const char *title;
+	EphyWindowPrivate *priv = window->priv;
 
-	if (window->priv->closing) return;
+	if (priv->closing) return;
 
-	title = ephy_tab_get_title (tab);
-
-	if (title)
-	{
-		gtk_window_set_title (GTK_WINDOW(window),
-				      title);
-	}
+	gtk_window_set_title (GTK_WINDOW(window), ephy_tab_get_title (tab));
 }
 
 static void
@@ -1535,30 +1551,79 @@ static void
 popup_menu_at_coords (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 		      gpointer user_data)
 {
-	GtkWidget *window = GTK_WIDGET (user_data);
-	EphyEmbedEvent *event;
+	EphyWindow *window = EPHY_WINDOW (user_data);
+	EphyWindowPrivate *priv = window->priv;
 	guint ux, uy;
 
-	event = g_object_get_data (G_OBJECT (window), "context_event");
-	g_return_if_fail (event != NULL);
+	g_return_if_fail (priv->context_event != NULL);
 
-	ephy_embed_event_get_coords (event, &ux, &uy);
+	ephy_embed_event_get_coords (priv->context_event, &ux, &uy);
 	*x = ux; *y = uy;
 
 	/* FIXME: better position the popup within the window bounds? */
-	ephy_gui_sanitise_popup_position (menu, window, x, y);
+	ephy_gui_sanitise_popup_position (menu, GTK_WIDGET (window), x, y);
 
 	*push_in = TRUE;
 }
 
-static void
-hide_embed_popup_cb (GtkWidget *popup,
-		     EphyWindow *window)
+static gboolean
+idle_unref_context_event (EphyWindow *window)
 {
+	EphyWindowPrivate *priv = window->priv;
+
+	LOG ("Idle unreffing context event %p", priv->context_event);
+
+	if (priv->context_event != NULL)
+	{
+		g_object_unref (priv->context_event);
+		priv->context_event = NULL;
+	}
+
+	priv->idle_worker = 0;
+	return FALSE;
+}
+
+static void
+set_context_event (EphyWindow *window,
+		   EphyEmbedEvent *event)
+{
+	EphyWindowPrivate *priv = window->priv;
+
+	if (priv->idle_worker != 0)
+	{
+		g_source_remove (priv->idle_worker);
+		priv->idle_worker = 0;
+	}
+
+	if (priv->context_event != NULL)
+	{
+		g_object_unref (priv->context_event);
+	}
+
+	priv->context_event = event != NULL ? g_object_ref (event) : NULL;
+}
+
+static void
+embed_popup_deactivate_cb (GtkWidget *popup,
+		          EphyWindow *window)
+{
+	EphyWindowPrivate *priv = window->priv;
+
+	LOG ("Deactivating popup menu");
+
 	enable_edit_actions_sensitivity (window);
 
 	g_signal_handlers_disconnect_by_func
-		(popup, G_CALLBACK (hide_embed_popup_cb), window);
+		(popup, G_CALLBACK (embed_popup_deactivate_cb), window);
+	
+	/* Unref the event from idle since we still need it
+	 * from the action callbacks which will run before idle.
+	 */
+	if (priv->idle_worker == 0 && priv->context_event != NULL)
+	{
+		priv->idle_worker =
+			g_idle_add ((GSourceFunc) idle_unref_context_event, window);
+	}
 }
 
 static char *
@@ -1574,7 +1639,7 @@ get_name_from_address_value (const GValue *value)
 		gnome_vfs_uri_unref (uri);
 	}
 
-	return name;
+	return name != NULL ? name : g_strdup ("");
 }
 
 static void
@@ -1760,12 +1825,10 @@ show_embed_popup (EphyWindow *window,
 	action = gtk_action_group_get_action (action_group, "OpenLinkInNewTab");
 	ephy_action_change_sensitivity_flags (action, SENS_FLAG_CONTEXT, !can_open_in_new);
 
-	g_object_set_data_full (G_OBJECT (window), "context_event",
-				g_object_ref (event),
-				(GDestroyNotify)g_object_unref);
+	set_context_event (window, event);
 
-	g_signal_connect (widget, "hide",
-			  G_CALLBACK (hide_embed_popup_cb), window);
+	g_signal_connect (widget, "deactivate",
+			  G_CALLBACK (embed_popup_deactivate_cb), window);
 
 	button = ephy_embed_event_get_button (event);
 	if (button == 0)
@@ -2029,7 +2092,7 @@ modal_alert_cb (EphyEmbed *embed,
 {
 	EphyWindowPrivate *priv = window->priv;
 	EphyTab *tab;
-	char *address;
+	const char *address;
 
 	tab = ephy_tab_for_embed (embed);
 	g_return_val_if_fail (tab != NULL, FALSE);
@@ -2045,9 +2108,8 @@ modal_alert_cb (EphyEmbed *embed,
 	gtk_window_present (GTK_WINDOW (window));
 
 	/* make sure the location entry shows the real URL of the tab's page */
-	address = ephy_embed_get_location (embed, TRUE);
-	ephy_toolbar_set_location (priv->toolbar, address);
-	g_free (address);
+	address = ephy_tab_get_address (tab);
+	ephy_toolbar_set_location (priv->toolbar, address, NULL);
 
 	/* don't suppress alert */
 	return FALSE;
@@ -2376,6 +2438,8 @@ ephy_window_dispose (GObject *object)
 
 		g_object_unref (priv->manager);
 		priv->manager = NULL;
+
+		set_context_event (window, NULL);
 	}
 
 	destroy_fullscreen_popup (window);
@@ -2544,6 +2608,7 @@ ephy_window_class_init (EphyWindowClass *klass)
 	widget_class->focus_in_event = ephy_window_focus_in_event;
 	widget_class->focus_out_event = ephy_window_focus_out_event;
 	widget_class->window_state_event = ephy_window_state_event;
+	widget_class->delete_event = ephy_window_delete_event;
 
 	g_object_class_install_property (object_class,
 					 PROP_ACTIVE_TAB,
@@ -2677,7 +2742,15 @@ ephy_window_open_link (EphyLink *link,
 		embed = ephy_tab_get_embed (tab);
 
 		ephy_embed_load_url (embed, address);
-		ephy_embed_activate (embed);
+
+		if (address == NULL || address[0] == '\0' || strcmp (address, "about:blank") == 0)
+		{
+			ephy_toolbar_activate_location (priv->toolbar);
+		}
+		else
+		{
+			gtk_widget_grab_focus (GTK_WIDGET (embed));
+		}
 
 		new_tab = tab;
 	}
@@ -2804,10 +2877,6 @@ ephy_window_init (EphyWindow *window)
 	egg_editable_toolbar_set_model
 		(EGG_EDITABLE_TOOLBAR (window->priv->toolbar), model);
 
-	g_signal_connect (window, "delete-event",
-			  G_CALLBACK (ephy_window_delete_event_cb),
-			  window);
-
 	/* other notifiers */
 	browse_with_caret_notifier (NULL, 0, NULL, window);
 	window->priv->browse_with_caret_notifier_id = eel_gconf_notification_add
@@ -3412,3 +3481,20 @@ ephy_window_get_is_print_preview (EphyWindow *window)
 
 	return window->priv->ppv_mode;
 }
+
+/**
+ * ephy_window_get_context_event:
+ * @window: an #EphyWindow
+ *
+ * Returns the #EphyEmbedEvent for the current context menu.
+ * Use this to get the event from the action callback.
+ *
+ * Return value: an #EphyEmbedEvent, or %NULL
+ **/
+EphyEmbedEvent *
+ephy_window_get_context_event (EphyWindow *window)
+{
+	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
+
+	return window->priv->context_event;
+}
